@@ -848,7 +848,7 @@ fillblock_rpm(struct blk *b, int id, struct seqdescr *sdesc, int nsdesc, struct
 }
 
 
-/* construct the block "id". Note that the tupel (idx, id) will
+/* construct the block "id". Note that the tuple (idx, id) will
  * only get bigger, so we use this to recycly no longer needed
  * blocks */
 
@@ -1449,8 +1449,10 @@ main(int argc, char **argv)
   if (strcmp(fnevr, (char *)d.nevr) != 0)
     {
       fprintf(stderr, "delta rpm made for %s, not %s\n", d.nevr, fnevr);
+      free(fnevr);
       exit(1);
     }
+  free(fnevr);
   if (!seqmatches)
     {
       fprintf(stderr, "rpm does not match the one used for creating the deltarpm\n");
@@ -1575,7 +1577,7 @@ main(int argc, char **argv)
 	  exit(1);
 	}
       rpmMD5Update(&wrmd5, d.h->intro, 16);
-      strncpy((char *)d.h->dp + d.payformatoff, "cpio", 4);
+      memcpy((char *)d.h->dp + d.payformatoff, "cpio", 4);
       if (fwrite(d.h->data, 16 * d.h->cnt + d.h->dcnt, 1, ofp) != 1)
 	{
 	  fprintf(stderr, "write error\n");
